#include "winres.h"

#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,3,0,0
 PRODUCTVERSION 1,3,0,0
 FILEFLAGSMASK 0x17L
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x2L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "FileDescription", "lib3ds Dynamic Link Library"
            VALUE "FileVersion", "1.3.0"
            VALUE "InternalName", "lib3ds"
            VALUE "LegalCopyright", "Copyright (C) 1996-2007 by Jan Eric Kyprianidis"
            VALUE "OriginalFilename", "lib3ds.dll"
            VALUE "ProductName", "lib3ds Dynamic Link Library"
            VALUE "ProductVersion", "1.3.0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END



